from .datasets import circuit_courts
from .datasets import discretized_functions
from .datasets import eastern_cities
from .datasets import federal_reserve_districts
from .datasets import iris
from .datasets import metropolises
from .datasets import mountains
from .datasets import states
from .datasets import students
from .datasets import western_cities
