from .demo import circle_to_diamond
from .demo import circle_to_double_shell
from .demo import projection_onto_diagonal
from .demo import reflect_over_vertical_axis
from .demo import rotate_45_ccw
from .demo import shear_vertically
from .utils import apply_func_to_basis
from .utils import get_2D_rotation_matrix
from .utils import get_parts_for_diagonalization_demo
from .utils import Rx
from .utils import Ry
from .utils import Rz
