from .histograms import distance_histogram_factory
from .histograms import pairs_histogram_factory
from .monte_carlo import p_ball_plot
from .pca_2D import cumulative_variance_plot_factory
from .pca_2D import scree_plot_factory
from .pca_3D import plot_3D_with_arrows
from .plotly import p_ball_plot_3d
from .plotly import plot_vectors_in_space
from .plotly import scatter_3D
from .plotly import scatter_3D_collinear
from .region import plot_circular_region_with_func_random_pts
from .region import plot_region_with_func
from .region import plot_region_with_func_random_pts
from .region import plot_region_with_two_basises
from .spectral_3D import plot_3D_spectral_demo
from .two_dimensional import abline
from .two_dimensional import timeseries_plot
from .vectors import does_function_preserve_addition
from .vectors import does_function_preserve_scalar_multiplication
from .vectors import plot_vector
